/*  
 * Actipro Software's WPF DataGrid / Editors Integration Add-on
 * http://actipro.codeplex.com/
 * 
 * Copyright (c) 2009-2013 Actipro Software LLC. 
 *  
 * This source code is subject to the terms of the Microsoft Public License (Ms-PL). 
 *  
 * Redistribution and use in source and binary forms, with or without modification, 
 * is permitted provided that redistributions of the source code retain the above 
 * copyright notices and this file header. 
 *  
 * Additional copyright notices should be appended to the list above. 
 * 
 * For details, see <http://www.opensource.org/licenses/ms-pl.html>. 
 *  
 * All other rights reserved. 
 *
 */  


using System;
using System.Diagnostics.CodeAnalysis;
using System.Windows;
using System.Windows.Media;
using ActiproSoftware.Windows.Controls.Editors.Interop.DataGrid.Primitives;
using ActiproSoftware.Windows.Controls.Editors.Parts;

namespace ActiproSoftware.Windows.Controls.Editors.Interop.DataGrid {

	/// <summary>
	/// Represents a data-bound column for use in a <c>DataGrid</c> that utilizes the <see cref="ColorEditBox"/>.
	/// </summary>
	public class DataGridColorColumn : DataGridTypeSpecificEditBoxColumnBase<Color?> {

		#region Dependency Properties

		/// <summary>
		/// Identifies the <see cref="ColorPreviewVisibility"/> dependency property.  This field is read-only.
		/// </summary>
		/// <value>The identifier for the <see cref="ColorPreviewVisibility"/> dependency property.</value>
		public static readonly DependencyProperty ColorPreviewVisibilityProperty = ColorEditBox.ColorPreviewVisibilityProperty.AddOwner(typeof(DataGridColorColumn),
			new FrameworkPropertyMetadata(Visibility.Visible, OnNotifyablePropertyValueChanged));

		/// <summary>
		/// Identifies the <see cref="IsAlphaComponentVisible"/> dependency property.  This field is read-only.
		/// </summary>
		/// <value>The identifier for the <see cref="IsAlphaComponentVisible"/> dependency property.</value>
		public static readonly DependencyProperty IsAlphaComponentVisibleProperty = ColorEditBox.IsAlphaComponentVisibleProperty.AddOwner(typeof(DataGridColorColumn),
			new FrameworkPropertyMetadata(true, OnNotifyablePropertyValueChanged));

		#endregion // Dependency Properties
		
		/////////////////////////////////////////////////////////////////////////////////////////////////////
		#region OBJECT
		/////////////////////////////////////////////////////////////////////////////////////////////////////

		/// <summary>
		/// Initializes the <see cref="DataGridColorColumn"/> class.
		/// </summary>
		[SuppressMessage("Microsoft.Performance", "CA1810:InitializeReferenceTypeStaticFieldsInline")]
		static DataGridColorColumn() {
			DropDownButtonVisibilityProperty.OverrideMetadata(typeof(DataGridColorColumn), new FrameworkPropertyMetadata(Visibility.Visible, DropDownButtonVisibilityProperty.DefaultMetadata.PropertyChangedCallback));
		}

		#endregion // OBJECT

		/////////////////////////////////////////////////////////////////////////////////////////////////////
		#region PUBLIC PROCEDURES
		/////////////////////////////////////////////////////////////////////////////////////////////////////

		/// <summary>
		/// Applies standard values to the specified target element.
		/// </summary>
		/// <param name="targetElement">The target element.</param>
		protected override void ApplyStandardValues(FrameworkElement targetElement) {
			if (targetElement is ColorEditBox) {
				this.ApplyValue(ColorPreviewVisibilityProperty, targetElement, ColorEditBox.ColorPreviewVisibilityProperty);
				this.ApplyValue(IsAlphaComponentVisibleProperty, targetElement, ColorEditBox.IsAlphaComponentVisibleProperty);
			}
			base.ApplyStandardValues(targetElement);
		}

		/// <summary>
		/// Gets or sets the visibility of the associated color preview.
		/// This is a dependency property.
		/// </summary>
		/// <value>
		/// The visibility of the associated color preview.
		/// The default value is <c>Visibility.Visible</c>.
		/// </value>
		public Visibility ColorPreviewVisibility {
			get { return (Visibility)this.GetValue(DataGridColorColumn.ColorPreviewVisibilityProperty); }
			set { this.SetValue(DataGridColorColumn.ColorPreviewVisibilityProperty, value); }
		}

		/// <summary>
		/// Gets the type of the edit box.
		/// </summary>
		/// <returns>The <see cref="ColorEditBox"/> type.</returns>
		protected override Type GetEditBoxType() {
			return typeof(ColorEditBox);
		}

		/// <summary>
		/// Gets or sets a value indicating whether the alpha component is visible.
		/// This is a dependency property.
		/// </summary>
		/// <value>
		/// <c>true</c> if the alpha component is visible; otherwise <c>false</c>.
		/// The default value is <c>true</c>.
		/// </value>
		/// <remarks>
		/// This property does not prevent a <see cref="ColorAlphaPart"/> from modifying the alpha component, this only affects
		/// the <see cref="ColorPart"/> and the automation text used for <see cref="ColorPartGroup"/> and <see cref="ColorEditBox"/>.
		/// </remarks>
		public bool IsAlphaComponentVisible {
			get { return (bool)this.GetValue(DataGridColorColumn.IsAlphaComponentVisibleProperty); }
			set { this.SetValue(DataGridColorColumn.IsAlphaComponentVisibleProperty, value); }
		}

		#endregion // PUBLIC PROCEDURES
	}
}
