/*  
 * Actipro Software's WPF DataGrid / Editors Integration Add-on
 * http://actipro.codeplex.com/
 * 
 * Copyright (c) 2009-2013 Actipro Software LLC. 
 *  
 * This source code is subject to the terms of the Microsoft Public License (Ms-PL). 
 *  
 * Redistribution and use in source and binary forms, with or without modification, 
 * is permitted provided that redistributions of the source code retain the above 
 * copyright notices and this file header. 
 *  
 * Additional copyright notices should be appended to the list above. 
 * 
 * For details, see <http://www.opensource.org/licenses/ms-pl.html>. 
 *  
 * All other rights reserved. 
 *
 */  


using System;
using System.Diagnostics.CodeAnalysis;
using System.Windows;
using ActiproSoftware.Windows.Controls.Editors.Interop.DataGrid.Primitives;

namespace ActiproSoftware.Windows.Controls.Editors.Interop.DataGrid {

	/// <summary>
	/// Represents a data-bound column for use in a <c>DataGrid</c> that utilizes the <see cref="DateTimeEditBox"/>.
	/// </summary>
	public class DataGridDateTimeColumn : DataGridRangeTypeSpecificEditBoxColumnBase<DateTime?> {

		#region Dependency Properties

		/// <summary>
		/// Identifies the <see cref="EditableParts"/> dependency property.  This field is read-only.
		/// </summary>
		/// <value>The identifier for the <see cref="EditableParts"/> dependency property.</value>
		public static readonly DependencyProperty EditablePartsProperty = DateTimeEditBox.EditablePartsProperty.AddOwner(typeof(DataGridDateTimeColumn),
			new FrameworkPropertyMetadata(DateTimeEditableParts.DateAndTime, OnNotifyablePropertyValueChanged));

		/// <summary>
		/// Identifies the <see cref="Format"/> dependency property.  This field is read-only.
		/// </summary>
		/// <value>The identifier for the <see cref="Format"/> dependency property.</value>
		public static readonly DependencyProperty FormatProperty = DateTimeEditBox.FormatProperty.AddOwner(typeof(DataGridDateTimeColumn),
			new FrameworkPropertyMetadata("g", OnNotifyablePropertyValueChanged));

		/// <summary>
		/// Identifies the <see cref="MonthCalendarStyle"/> dependency property.  This field is read-only.
		/// </summary>
		/// <value>The identifier for the <see cref="MonthCalendarStyle"/> dependency property.</value>
		public static readonly DependencyProperty MonthCalendarStyleProperty = DateTimeEditBox.MonthCalendarStyleProperty.AddOwner(typeof(DataGridDateTimeColumn),
			new FrameworkPropertyMetadata(null, OnNotifyablePropertyValueChanged));

		#endregion // Dependency Properties

		/////////////////////////////////////////////////////////////////////////////////////////////////////
		#region OBJECT
		/////////////////////////////////////////////////////////////////////////////////////////////////////

		/// <summary>
		/// Initializes the <see cref="DataGridDateTimeColumn"/> class.
		/// </summary>
		[SuppressMessage("Microsoft.Performance", "CA1810:InitializeReferenceTypeStaticFieldsInline")]
		static DataGridDateTimeColumn() {
			DropDownButtonVisibilityProperty.OverrideMetadata(typeof(DataGridDateTimeColumn), new FrameworkPropertyMetadata(Visibility.Visible, DropDownButtonVisibilityProperty.DefaultMetadata.PropertyChangedCallback));
			SpinBehaviorProperty.OverrideMetadata(typeof(DataGridDateTimeColumn), new FrameworkPropertyMetadata(SpinBehavior.Wrap, SpinBehaviorProperty.DefaultMetadata.PropertyChangedCallback));
		}

		#endregion // OBJECT

		/////////////////////////////////////////////////////////////////////////////////////////////////////
		#region PUBLIC PROCEDURES
		/////////////////////////////////////////////////////////////////////////////////////////////////////

		/// <summary>
		/// Applies standard values to the specified target element.
		/// </summary>
		/// <param name="targetElement">The target element.</param>
		protected override void ApplyStandardValues(FrameworkElement targetElement) {
			if (targetElement is DateTimeEditBox) {
				this.ApplyValue(EditablePartsProperty, targetElement, DateTimeEditBox.EditablePartsProperty);
				this.ApplyValue(FormatProperty, targetElement, DateTimeEditBox.FormatProperty);
				this.ApplyValue(MonthCalendarStyleProperty, targetElement, DateTimeEditBox.MonthCalendarStyleProperty);
			}
			base.ApplyStandardValues(targetElement);
		}

		/// <summary>
		/// Gets or sets the parts that are editable.
		/// This is a dependency property.
		/// </summary>
		/// <value>
		/// The parts that are editable.
		/// The default value is <c>DateTimeEditableParts.DateAndTime</c>.
		/// </value>
		public DateTimeEditableParts EditableParts {
			get { return (DateTimeEditableParts)this.GetValue(DataGridDateTimeColumn.EditablePartsProperty); }
			set { this.SetValue(DataGridDateTimeColumn.EditablePartsProperty, value); }
		}

		/// <summary>
		/// Gets or sets the date/time format string.
		/// This is a dependency property.
		/// </summary>
		/// <value>
		/// The date/time format string.
		/// The default value is <c>"g"</c>.
		/// </value>
		public string Format {
			get { return (string)this.GetValue(DataGridDateTimeColumn.FormatProperty); }
			set { this.SetValue(DataGridDateTimeColumn.FormatProperty, value); }
		}

		/// <summary>
		/// Gets the type of the edit box.
		/// </summary>
		/// <returns>The <see cref="DateTimeEditBox"/> type.</returns>
		protected override Type GetEditBoxType() {
			return typeof(DateTimeEditBox);
		}

		/// <summary>
		/// Gets or sets the <see cref="Style"/> applied to the <see cref="MonthCalendar"/> used in default drop-down content.
		/// This is a dependency property.
		/// </summary>
		/// <value>
		/// The <see cref="Style"/> applied to the <see cref="MonthCalendar"/> used in default drop-down content.
		/// The default value is <c>null</c>.
		/// </value>
		public Style MonthCalendarStyle {
			get { return (Style)this.GetValue(DataGridDateTimeColumn.MonthCalendarStyleProperty); }
			set { this.SetValue(DataGridDateTimeColumn.MonthCalendarStyleProperty, value); }
		}

		#endregion // PUBLIC PROCEDURES
	}
}
