/*  
 * Actipro Software's WPF DataGrid / Editors Integration Add-on
 * http://actipro.codeplex.com/
 * 
 * Copyright (c) 2009-2013 Actipro Software LLC. 
 *  
 * This source code is subject to the terms of the Microsoft Public License (Ms-PL). 
 *  
 * Redistribution and use in source and binary forms, with or without modification, 
 * is permitted provided that redistributions of the source code retain the above 
 * copyright notices and this file header. 
 *  
 * Additional copyright notices should be appended to the list above. 
 * 
 * For details, see <http://www.opensource.org/licenses/ms-pl.html>. 
 *  
 * All other rights reserved. 
 *
 */  


using System;
using System.Windows;
using System.Windows.Controls;
using ActiproSoftware.Windows.Controls.Editors.Interop.DataGrid.Primitives;

namespace ActiproSoftware.Windows.Controls.Editors.Interop.DataGrid {

	/// <summary>
	/// Represents a data-bound column for use in a <c>DataGrid</c> that utilizes the <see cref="GuidEditBox"/>.
	/// </summary>
	public class DataGridGuidColumn : DataGridTypeSpecificEditBoxColumnBase<Guid?> {

		#region Dependency Properties

		/// <summary>
		/// Identifies the <see cref="Format"/> dependency property.  This field is read-only.
		/// </summary>
		/// <value>The identifier for the <see cref="Format"/> dependency property.</value>
		public static readonly DependencyProperty FormatProperty = GuidEditBox.FormatProperty.AddOwner(typeof(DataGridGuidColumn),
			new FrameworkPropertyMetadata("D", OnNotifyablePropertyValueChanged));

		/// <summary>
		/// Identifies the <see cref="NewGuidButtonVisibility"/> dependency property.  This field is read-only.
		/// </summary>
		/// <value>The identifier for the <see cref="NewGuidButtonVisibility"/> dependency property.</value>
		public static readonly DependencyProperty NewGuidButtonVisibilityProperty = GuidEditBox.NewGuidButtonVisibilityProperty.AddOwner(typeof(DataGridGuidColumn),
			new FrameworkPropertyMetadata(Visibility.Collapsed, OnNotifyablePropertyValueChanged));

		#endregion // Dependency Properties

		/////////////////////////////////////////////////////////////////////////////////////////////////////
		#region PUBLIC PROCEDURES
		/////////////////////////////////////////////////////////////////////////////////////////////////////

		/// <summary>
		/// Applies standard values to the specified target element.
		/// </summary>
		/// <param name="targetElement">The target element.</param>
		protected override void ApplyStandardValues(FrameworkElement targetElement) {
			if (targetElement is GuidEditBox) {
				this.ApplyValue(FormatProperty, targetElement, GuidEditBox.FormatProperty);
				this.ApplyValue(NewGuidButtonVisibilityProperty, targetElement, GuidEditBox.NewGuidButtonVisibilityProperty);
			}
			base.ApplyStandardValues(targetElement);
		}

		/// <summary>
		/// Gets or sets the GUID format string.
		/// This is a dependency property.
		/// </summary>
		/// <value>
		/// The GUID format string.
		/// The default value is <c>"D"</c>.
		/// </value>
		public string Format {
			get { return (string)this.GetValue(DataGridGuidColumn.FormatProperty); }
			set { this.SetValue(DataGridGuidColumn.FormatProperty, value); }
		}

		/// <summary>
		/// Gets the type of the edit box.
		/// </summary>
		/// <returns></returns>
		/// <returns>The <see cref="GuidEditBox"/> type.</returns>
		protected override Type GetEditBoxType() {
			return typeof(GuidEditBox);
		}

		/// <summary>
		/// Gets or sets the visibility of the associated new Guid <see cref="Button"/>.
		/// This is a dependency property.
		/// </summary>
		/// <value>
		/// The visibility of the associated new Guid <see cref="Button"/>.
		/// The default value is <c>Visibility.Collapsed</c>.
		/// </value>
		public Visibility NewGuidButtonVisibility {
			get { return (Visibility)this.GetValue(DataGridGuidColumn.NewGuidButtonVisibilityProperty); }
			set { this.SetValue(DataGridGuidColumn.NewGuidButtonVisibilityProperty, value); }
		}

		#endregion // PUBLIC PROCEDURES
	}
}
