/*  
 * Actipro Software's WPF DataGrid / Editors Integration Add-on
 * http://actipro.codeplex.com/
 * 
 * Copyright (c) 2009-2013 Actipro Software LLC. 
 *  
 * This source code is subject to the terms of the Microsoft Public License (Ms-PL). 
 *  
 * Redistribution and use in source and binary forms, with or without modification, 
 * is permitted provided that redistributions of the source code retain the above 
 * copyright notices and this file header. 
 *  
 * Additional copyright notices should be appended to the list above. 
 * 
 * For details, see <http://www.opensource.org/licenses/ms-pl.html>. 
 *  
 * All other rights reserved. 
 *
 */  


using System;
using System.Diagnostics.CodeAnalysis;
using System.Windows;
using ActiproSoftware.Windows.Controls.Editors.Interop.DataGrid.Primitives;

namespace ActiproSoftware.Windows.Controls.Editors.Interop.DataGrid {

	/// <summary>
	/// Represents a data-bound column for use in a <c>DataGrid</c> that utilizes the <see cref="TimeSpanEditBox"/>.
	/// </summary>
	public class DataGridTimeSpanColumn : DataGridRangeTypeSpecificEditBoxColumnBase<TimeSpan?> {

		#region Dependency Properties

		/// <summary>
		/// Identifies the <see cref="EditableParts"/> dependency property.  This field is read-only.
		/// </summary>
		/// <value>The identifier for the <see cref="EditableParts"/> dependency property.</value>
		public static readonly DependencyProperty EditablePartsProperty = TimeSpanEditBox.EditablePartsProperty.AddOwner(typeof(DataGridTimeSpanColumn),
			new FrameworkPropertyMetadata(TimeSpanEditableParts.All, OnNotifyablePropertyValueChanged));

		/// <summary>
		/// Identifies the <see cref="Format"/> dependency property.  This field is read-only.
		/// </summary>
		/// <value>The identifier for the <see cref="Format"/> dependency property.</value>
		public static readonly DependencyProperty FormatProperty = TimeSpanEditBox.FormatProperty.AddOwner(typeof(DataGridTimeSpanColumn),
			new FrameworkPropertyMetadata("g", OnNotifyablePropertyValueChanged));

		#endregion // Dependency Properties

		/////////////////////////////////////////////////////////////////////////////////////////////////////
		#region OBJECT
		/////////////////////////////////////////////////////////////////////////////////////////////////////

		/// <summary>
		/// Initializes the <see cref="DataGridTimeSpanColumn"/> class.
		/// </summary>
		[SuppressMessage("Microsoft.Performance", "CA1810:InitializeReferenceTypeStaticFieldsInline")]
		static DataGridTimeSpanColumn() {
			SpinBehaviorProperty.OverrideMetadata(typeof(DataGridTimeSpanColumn), new FrameworkPropertyMetadata(SpinBehavior.Wrap, SpinBehaviorProperty.DefaultMetadata.PropertyChangedCallback));
		}

		#endregion // OBJECT

		/////////////////////////////////////////////////////////////////////////////////////////////////////
		#region PUBLIC PROCEDURES
		/////////////////////////////////////////////////////////////////////////////////////////////////////

		/// <summary>
		/// Applies standard values to the specified target element.
		/// </summary>
		/// <param name="targetElement">The target element.</param>
		protected override void ApplyStandardValues(FrameworkElement targetElement) {
			if (targetElement is TimeSpanEditBox) {
				this.ApplyValue(EditablePartsProperty, targetElement, TimeSpanEditBox.EditablePartsProperty);
				this.ApplyValue(FormatProperty, targetElement, TimeSpanEditBox.FormatProperty);
			}
			base.ApplyStandardValues(targetElement);
		}

		/// <summary>
		/// Gets or sets the parts that are editable.
		/// This is a dependency property.
		/// </summary>
		/// <value>
		/// The parts that are editable.
		/// The default value is <c>TimeSpanEditableParts.All</c>.
		/// </value>
		public TimeSpanEditableParts EditableParts {
			get { return (TimeSpanEditableParts)this.GetValue(DataGridTimeSpanColumn.EditablePartsProperty); }
			set { this.SetValue(DataGridTimeSpanColumn.EditablePartsProperty, value); }
		}

		/// <summary>
		/// Gets or sets the time span format string.
		/// This is a dependency property.
		/// </summary>
		/// <value>
		/// The time span format string.
		/// The default value is <c>"g"</c>.
		/// </value>
		public string Format {
			get { return (string)this.GetValue(DataGridTimeSpanColumn.FormatProperty); }
			set { this.SetValue(DataGridTimeSpanColumn.FormatProperty, value); }
		}

		/// <summary>
		/// Gets the type of the edit box.
		/// </summary>
		/// <returns>The <see cref="TimeSpanEditBox"/> type.</returns>
		protected override Type GetEditBoxType() {
			return typeof(TimeSpanEditBox);
		}

		#endregion // PUBLIC PROCEDURES
	}
}
